@@ -12,7 +12,7 @@ use warnings;
 
 =head1 NAME
 
-IMDB::Local - The great new IMDB::Local!
+IMDB::Local - Tools to dowload and manage a local copy of the IMDB list files in a database.
 
 =head1 VERSION
 
@@ -22,16 +22,21 @@ Version 1.0
 
 our $VERSION = '1.0';
 
-
 =head1 SYNOPSIS
 
 Quick summary of what the module does.
 
 Perhaps a little code snippet.
 
-    use IMDB::Local;
+    my $foo = new IMDB::Local('imdbDir'  =>  "./imdb-data",
+                              'listsDir' =>  "./imdb-data/lists",
+                              'showProgressBar' => 1);
 
-    my $foo = IMDB::Local->new();
+    for my $type ( $foo->listTypes() ) {
+        if ( $foo->importList($type) != 0 ) {
+           warn("$type import failed, check $foo->{imdbDir}/stage-$type.log");
+        }
+    }
     ...
 
 =head1 EXPORT
@@ -95,6 +100,28 @@ sub new
     return($self);
 }
 
+#sub openDB
+#{
+#    my ($self)=@_;
+#
+#    my $DB=new IMDB::Local::DB(database=>"$self->{imdbDir}/imdb.db");
+#
+#    if ( !$DB->connect() ) {
+#	carp "imdbdb connect failed:$DBI::errstr";
+#    }
+#    $self->{DB}=$DB;
+#
+#    return($DB);
+#}
+#
+#sub closeDB
+#{
+#    my ($self)=@_;
+#
+#    $self->{DB}->disconnect();
+#    undef $self->{DB};
+#}
+
 =head2 listTypes
 
 Returns an array of list files supported (currently 'movies', 'directors', 'actors', 'actresses', 'genres', 'ratings', 'keywords', 'plot')
@@ -182,71 +209,6 @@ sub closeMaybeGunzip($$)
     }
 }
 
-
-# Convert a title into a searchtitle by lowercasing,
-# making it ASCII and removing punctuation.
-#
-sub MakeSearchtitle($;$;$) {
-  my ($DB, $str, $debug) = @_;
-  return lc RemovePunctuation($DB, lc( CharsetMap( $str, $debug ) ), $debug );
-}
-
-
-use Text::Unidecode;
-
-#   All characters outside the ASCII range (0x00-0x7F) are replaced by ASCII equivalents,
-#    using function Text::Unidecode::unidecode
-#
-sub CharsetMap($) {
-  my ($str, $debug) = @_;
-  
-  # do replacements that unidecode doesn't know about (or does wrong)
-  ### IT WOULD BE NICE IF THESE WERE IN A TABLE
-  $str =~ s/\x{0133}/ij/g;     # 'ij' -> ij             ("" in unidecode)
-  $str =~ s/\x{20ac}/EUR/g;    # euro symbol -> EUR     (EU in unidecode)
-  $str =~ s/\x{2122}/TM/g;     # trademark symbol -> TM ("" in unidecode)
-  $str =~ s/\x{a3}/GBP/g;      # pound sign -> GBP (PS in unidecode)
-  
-  # now do the real decode
-  $str = unidecode($str);
-        
-  #print "[$str]\n" if ($debug);
- 
-  return ($str);
-}
-
-
-my @punctuation;
-
-# Function that removes all punctuation and whitespace from a string.
-#  '&' is converted to 'and' along the way
-sub RemovePunctuation($;$;$) {
-    my ($DB, $str, $debug) = @_;
-
-    # Load the array of hashes that contain the punctuation
-    # replacements in priority order
-    if ( !@punctuation ) {
-        my @plist = @{$DB->select2Matrix("select priority,pattern,replacement from Punctuation order by priority")};
-        my $cnt = 0;
-        foreach my $p (@plist) {
-            my $pattern = $p->[1];
-            my $compiled = qr/$pattern/i;
-            $punctuation[$cnt]{origpattern} = $pattern;
-            $punctuation[$cnt]{pattern}     = $compiled;
-            $punctuation[$cnt]{replacement} = $p->[2];
-            $cnt++;
-        }
-    }
-
-    foreach my $ref (@punctuation) {
-        #print "[$str] $ref->{origpattern} " if ($debug);
-        $str =~ s/$ref->{pattern}/$ref->{replacement}/g;
-        #print "[$str]\n" if ($debug);
-    }
-    #print "<$str>\n--------\n" if ($debug);
-    return ($str);
-}
-
 sub decodeImdbKey($$$)
 {
     my ($self, $DB, $dbkey, $year, $titleID)=@_;
@@ -257,7 +219,6 @@ sub decodeImdbKey($$$)
     $hash{series}=0;
     $hash{episode}=0;
     
-
     # drop episode information - ex: "Studio One" (1948) {Twelve Angry Men (#7.1)}
     if ( $dbkey=~s/\s*\{([^\}]+)\}//o ) {
 	my $s=$1;
@@ -265,6 +226,7 @@ sub decodeImdbKey($$$)
 	    $hash{series}=$1;
 	    $hash{episode}=$2;
 	    $hash{title}=$s;
+	    #print "title: $s\n";
 
 	    # attempt to locate parentId matching series title
 	    my $parentKey=$dbkey;
@@ -275,9 +237,11 @@ sub decodeImdbKey($$$)
 		$hash{parentId}=$self->{seriesKeys}->{$parentKey};
 	    }
 	}
+	else {
+	    $hash{title}=$s;
+	}
     }
 		
-    # todo - this would make things easier
     # change double-quotes around title to be (made-for-tv) suffix instead 
     if ( $dbkey=~s/^\"//o && $dbkey=~s/\" \(/ \(/o) {
 	if ( $dbkey=~s/\s+\(mini\)$//o ) {
@@ -328,7 +292,7 @@ sub decodeImdbKey($$$)
     }
     
     if ( $title=~s/\s+\((\d\d\d\d)\)$//o ||
-	 $title=~s/\s+\((\d\d\d\d)\/[IVX]+\)$//o ) {
+	 $title=~s/\s+\((\d\d\d\d)\/[IVXL]+\)$//o ) {
 	# over-ride with what is given
 	if ( defined($year) ) {
 	    $hash{year}=$year;
@@ -338,7 +302,7 @@ sub decodeImdbKey($$$)
 	}
     }
     elsif ( $title=~s/\s+\((\?\?\?\?)\)$//o ||
-	    $title=~s/\s+\((\?\?\?\?)\/[IVX]+\)$//o ) {
+	    $title=~s/\s+\((\?\?\?\?)\/[IVXL]+\)$//o ) {
 	# over-ride with what is given
 	if ( defined($year) ) {
 	    $hash{year}=$year;
@@ -349,6 +313,7 @@ sub decodeImdbKey($$$)
     }
     else {
 	$self->error("movie list format failed to decode year from title '$title'");
+	
 	# over-ride with what is given
 	if ( defined($year) ) {
 	    $hash{year}=$year;
@@ -359,18 +324,24 @@ sub decodeImdbKey($$$)
     }
     $title=~s/(.*),\s*(The|A|Une|Las|Les|Los|L\'|Le|La|El|Das|De|Het|Een)$/$2 $1/og;
 
-    if ( !defined($hash{title}) ) {
-	$hash{title}=$title;
-	$hash{searchTitle}=MakeSearchtitle($DB, $hash{title}, 0);
-
-	# todo - is this more useful ?
-	#$hash{searchTitleWithYear}=MakeSearchtitle($DB, $title."(".$hash{year}.")", 0);
+    # leave searchtitle empty for tv series'
+    if ( $hash{series} != 0 ) {
+	$hash{searchTitle}='';
     }
     else {
-	$hash{searchTitle}=MakeSearchtitle($DB, $title."(".$hash{year}.")", 0);
+	if ( !defined($hash{title}) ) {
+	    $hash{title}=$title;
+	    $hash{searchTitle}=$DB->makeSearchableTitle($title, 0);
 
-	# todo - is this more useful ?
-	#$hash{searchTitleWithYear}=MakeSearchtitle($DB, $title."(".$hash{year}.")", 0);
+	    # todo - is this more useful ?
+	    #$hash{searchTitleWithYear}=MakeSearchtitle($DB, $title."(".$hash{year}.")", 0);
+	}
+	else {
+	    $hash{searchTitle}=$DB->makeSearchableTitle($title."(".$hash{year}.")", 0);
+	    
+	    # todo - is this more useful ?
+	    #$hash{searchTitleWithYear}=$DB->makeSearchableTitle($title."(".$hash{year}.")", 0);
+	}
     }
 
     return(\%hash);
@@ -420,19 +391,22 @@ sub importMovies($$$$)
     #$DB->runSQL("BEGIN TRANSACTION");
 
     my $count=0;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Titles VALUES (?,?,?,?,?,?,?,?)');
+    my $tableInsert_sth=$DB->prepare('INSERT INTO Titles (TitleID, SearchTitle, Title, QualifierTypeID, Year, ParentID, Series, Episode) VALUES (?,?,?,?,?,?,?,?)');
 
+    my $potentialEntries=0;
+    
     while(<$fh>) {
 	$lineCount++;
 	my $line=$_;
-	#print "read line $lineCount:$line\n";
 
 	# end is line consisting of only '-'
 	last if ( $line=~m/^\-\-\-\-\-\-\-+/o );
 
 	next if ( $line=~m/\{\{SUSPENDED\}\}/o );
-
 	$line=~s/\n$//o;
+	
+	#print "read line $lineCount:$line\n";
+	$potentialEntries++;
 
 	my $tab=index($line, "\t");
 	if ( $tab != -1 ) {
@@ -492,7 +466,6 @@ sub importMovies($$$$)
 	}
 	else {
 	    $self->error("$file:$lineCount: unrecognized format (missing tab)");
-	    $next_update=$progress->update($count) if ($self->{showProgressBar});
 	}
     }
     #$DB->runSQL("END TRANSACTION");
@@ -500,9 +473,10 @@ sub importMovies($$$$)
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Movies found ".withThousands($count)." in ".
-			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
+    $DB->commit();
     return($count);
 }
 
@@ -550,7 +524,8 @@ sub importGenres($$$$)
     #$DB->runSQL("BEGIN TRANSACTION");
 
     my $count=0;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Titles2Genres VALUES (?,?)');
+    my $potentialEntries=0;
+    my $tableInsert_sth=$DB->prepare('INSERT INTO Titles2Genres (TitleID, GenreID) VALUES (?,?)');
 
     while(<$fh>) {
 	$lineCount++;
@@ -559,6 +534,9 @@ sub importGenres($$$$)
 
 	# end is line consisting of only '-'
 	last if ( $line=~m/^\-\-\-\-\-\-\-+/o );
+	next if ( $line=~m/\s*\{\{SUSPENDED\}\}/o);
+
+	$potentialEntries++;
 
 	$line=~s/\n$//o;
 
@@ -566,13 +544,6 @@ sub importGenres($$$$)
 	if ( $tab != -1 ) {
 	    my $mkey=substr($line, 0, $tab);
 
-	    next if ($mkey=~m/\s*\{\{SUSPENDED\}\}/o);
-
-	    # Genres
-	    # don't see what these are...?
-	    # ignore {{SUSPENDED}}
-	    $mkey=~s/\s*\{\{SUSPENDED\}\}//o;
-
 	    # ignore {Twelve Angry Men (1954)}
 	    # TODO - do we want this ?
 	    #$mkey=~s/\s*\{[^\}]+\}//go;
@@ -614,7 +585,6 @@ sub importGenres($$$$)
 	}
 	else {
 	    $self->error("$file:$lineCount: unrecognized format (missing tab)");
-	    $next_update=$progress->update($count) if ($self->{showProgressBar});
 	}
     }
     #$DB->runSQL("END TRANSACTION");
@@ -622,9 +592,10 @@ sub importGenres($$$$)
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Genres found ".withThousands($count)." in ".
-			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
+    $DB->commit();
     return($count);
 }
 
@@ -635,10 +606,14 @@ sub importActors($$$$)
 
     if ( $whichCastType eq "Actors" ) {
 	if ( $DB->table_row_count('Actors') > 0 ||
-	     $DB->table_row_count('Titles2Actors') > 0 ) {
+	     $DB->table_row_count('Titles2Actors') > 0 ||
+	     $DB->table_row_count('Titles2Hosts') > 0 ||
+	     $DB->table_row_count('Titles2Narrators') > 0 ) {
 	    $self->status("clearing previously loaded data..");
-	    $DB->runSQL("DELETE from Actors");
-	    $DB->runSQL("DELETE from Titles2Actors");
+	    $DB->table_clear('Actors');
+	    $DB->table_clear('Titles2Actors');
+	    $DB->table_clear('Titles2Hosts');
+	    $DB->table_clear('Titles2Narrators');
 	}
     }
 
@@ -666,6 +641,7 @@ sub importActors($$$$)
     $progress->minor(0) if ($self->{showProgressBar});
     $progress->max_update_rate(1) if ($self->{showProgressBar});
     my $next_update=0;
+    
     while(<$fh>) {
 	$lineCount++;
 	if ( m/^$header/ ) {
@@ -701,10 +677,17 @@ sub importActors($$$$)
     my $cur_name;
     my $count=0;
     my $castNames=0;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Actors VALUES (?,?,?)');
-    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Actors VALUES (?,?,?,?,?)');
-    my $cur_actorId=$DB->select2Scalar('select count(1) from Actors');
+    my $tableInsert_sth1=$DB->prepare('INSERT INTO Actors           (ActorID, SearchName, Name) VALUES (?,?,?)');
+    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Hosts     (TitleID, ActorID) VALUES (?,?)');
+    my $tableInsert_sth3=$DB->prepare('INSERT INTO Titles2Narrators (TitleID, ActorID) VALUES (?,?)');
+    my $tableInsert_sth4=$DB->prepare('INSERT INTO Titles2Actors    (TitleID, ActorID, Billing) VALUES (?,?,?)');
+    
+    my $cur_actorId=$DB->select2Scalar('Select MAX(ActorID) from Actors');
+    if ( !defined($cur_actorId) ) {
+	$cur_actorId=0;
+    }
 
+    my $potentialEntries=0;
     while(<$fh>) {
 	$lineCount++;
 	my $line=$_;
@@ -719,7 +702,8 @@ sub importActors($$$$)
 	# try ignoring these
 	next if ($line=~m/\s*\{\{SUSPENDED\}\}/o);
 
-	my $billing=0;
+	$potentialEntries++;
+	my $billing=9999;
 	
 	# actors or actresses
 	if ( $line=~s/\s*<(\d+)>//o ) {
@@ -735,7 +719,7 @@ sub importActors($$$$)
 
 	    my $c=$cur_name;
 	    $c=~s/\s*\([IVX]+\)//o;
-	    $tableInsert_sth->execute($cur_actorId, MakeSearchtitle($DB, $c, 0), $cur_name);
+	    $tableInsert_sth1->execute($cur_actorId, $DB->makeSearchableTitle($c, 0), $cur_name);
 
 	    if ( $self->{showProgressBar} ) {
 		# re-adjust target so progress bar doesn't seem too wonky
@@ -765,15 +749,11 @@ sub importActors($$$$)
 	    # ignore character name
 	}
 	
-	# BUG
-  # [honir] this is wrong - this puts cast from all the episodes as though they are in the entire series!
-	# ##ignore {Twelve Angry Men (1954)}
-	# TODO - I REMOVED THIS, should we ?
-	#$line=~s/\s*\{[^\}]+\}//o;
-
+	# TODO - do we want to just ignore these ?
 	if ( $line=~s/\s*\(aka ([^\)]+)\).*$//o ) {
 	    #$attrs=$1;
 	}
+	
 	# TODO - what are we ignoring here ?
 	if ( $line=~s/  (\(.*)$//o ) {
 	    #$attrs=$1;
@@ -781,7 +761,9 @@ sub importActors($$$$)
 	$line=~s/^\s+//og;
 	$line=~s/\s+$//og;
 
+	# TODO - does this exist ?
 	if ( $line=~s/\s+Narrator$//o ) {
+	    $self->error("extra narrator on line: $lineCount");
 	    # TODO - do we want to store this ? Does it actually occur ?
 	    # ignore
 	}
@@ -789,11 +771,19 @@ sub importActors($$$$)
 	#if ( $line=~s/\s*\([A-Z]+\)$//o ) {
 	#}
 
-	if ( $self->{imdbMovie2DBKey}->{$line} ) {
+	my $titleID=$self->{imdbMovie2DBKey}->{$line};
+	if ( $titleID ) {
+	    if ( $isHost ) {
+		$tableInsert_sth2->execute($titleID, $cur_actorId);
+	    }
+	    if ( $isNarrator ) {
+		$tableInsert_sth3->execute($titleID, $cur_actorId);
+	    }
+	    if ( !$isHost && !$isNarrator ) {
+		$tableInsert_sth4->execute($titleID, $cur_actorId, $billing);
+	    }
 	    
-	    $tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$line}, $cur_actorId, $billing, $isHost, $isNarrator);
 	    $count++;
-
 	    if ( ($count % 50000) == 0 ) {
 		$DB->commit();
 	    }
@@ -805,10 +795,11 @@ sub importActors($$$$)
     $progress->update($castCountEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing $whichCastType found ".withThousands($castNames)." names, ".
-			  withThousands($count)." titles in ".withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($count)." titles in ".withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
     
     closeMaybeGunzip($file, $fh);
 
+    $DB->commit();
     return($castNames);
 }
 
@@ -822,8 +813,8 @@ sub importDirectors($$$)
     if ( $DB->table_row_count('Directors') > 0 ||
 	 $DB->table_row_count('Titles2Directors') > 0 ) {
 	$self->status("clearing previously loaded data..");
-	$DB->runSQL("DELETE from Directors");
-	$DB->runSQL("DELETE from Titles2Directors");
+	$DB->table_clear('Directors');
+	$DB->table_clear('Titles2Directors');
     }
 
     my $fh = openMaybeGunzip($file) || return(-2);
@@ -871,20 +862,27 @@ sub importDirectors($$$)
     my $castNames=0;
     my %found;
     my $directorCount=0;
+    my $potentialEntries=0;
 
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Directors VALUES (?,?,?)');
-    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Directors VALUES (?,?)');
+    my $tableInsert_sth=$DB->prepare('INSERT INTO Directors (DirectorID, SearchName, Name) VALUES (?,?,?)');
+    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Directors (TitleID, DirectorID) VALUES (?,?)');
     while(<$fh>) {
 	$lineCount++;
+	
+	#last if ( $lineCount > 10000);
 	my $line=$_;
 	$line=~s/\n$//o;
 	#$self->status("read line $lineCount:$line");
 
 	# end is line consisting of only '-'
 	last if ( $line=~m/^\-\-\-\-\-\-\-+/o );
-	
 	next if ( length($line) == 0 );
-
+	
+	# try ignoring these
+	next if ($line=~m/\s*\{\{SUSPENDED\}\}/o);
+	
+	$potentialEntries++;
+	
 	if ( $line=~s/^([^\t]+)\t+//o ) {
 	    $cur_name=$1;
 	    $castNames++;
@@ -901,29 +899,33 @@ sub importDirectors($$$)
 	    }
 	}
 	
-	# try ignoring these
-	next if ($line=~m/\s*\{\{SUSPENDED\}\}/o);
-
 	# BUG
-  # [honir] this is wrong - this puts cast from all the episodes as though they are in the entire series!
 	# ##ignore {Twelve Angry Men (1954)}
-	$line=~s/\s*\{[^\}]+\}//o;
+	#$line=~s/\s*\{[^\}]+\}//o;
 
-	# TODO - what are we ignoring here ?
+	# sometimes there are extra bits of info attached at the end of lines, we'll ignore these
+	#
+	# examples:
+	# Deszcz (1997)  (as Tomasz Baginski)
+	# Adventures of Modest Mouse (2008)  (co-director)
+	# Vida (2010)  (collaborating director)
+	# Rex Harrison Presents Stories of Love (1974) (TV)  (segment "Epicac")
 	if ( $line=~s/  (\(.*)$//o ) {
 	    # $attrs=$1;
 	}
+	
 	$line=~s/^\s+//og;
 	$line=~s/\s+$//og;
 
 	if ( $self->{imdbMovie2DBKey}->{$line} ) {
 
 	    if ( !defined($found{$cur_name}) ) {
-		$found{$cur_name}=$directorCount++;
+		$directorCount++;
+		$found{$cur_name}=$directorCount;
 
 		my $c=$cur_name;
 		$c=~s/\s*\([IVX]+\)//o;
-		$tableInsert_sth->execute($directorCount, MakeSearchtitle($DB, $c, 0), $cur_name);
+		$tableInsert_sth->execute($directorCount, $DB->makeSearchableTitle($c, 0), $cur_name);
 	    }
 
 	    $tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$line}, $found{$cur_name});
@@ -932,14 +934,18 @@ sub importDirectors($$$)
 		$DB->commit();
 	    }
 	}
+	else {
+	    $self->error("$lineCount: unable to match title key '$line'");
+	}
     }
     $progress->update($castCountEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Directors found ".withThousands($castNames)." names, ".
-			  withThousands($count)." titles in ".withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($count)." titles in ".withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
     
     closeMaybeGunzip($file, $fh);
 
+    $DB->commit();
     return($castNames);
 }
 
@@ -951,7 +957,7 @@ sub importRatings($$)
 
     if ( $DB->table_row_count('Ratings') > 0 ) {
 	$self->status("clearing previously loaded data..");
-	$DB->runSQL("DELETE from Ratings");
+	$DB->table_clear('Ratings');
     }
 
     my $fh = openMaybeGunzip($file) || return(-2);
@@ -988,7 +994,8 @@ sub importRatings($$)
 
     my $countImported=0;
     my $count=0;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Ratings VALUES (?,?,?,?)');
+    my $potentialEntries=0;
+    my $tableInsert_sth=$DB->prepare('INSERT INTO Ratings (TitleID, Distribution, Votes, Rank) VALUES (?,?,?,?)');
     while(<$fh>) {
 	$lineCount++;
 	my $line=$_;
@@ -1001,6 +1008,8 @@ sub importRatings($$)
 	# end is line consisting of only '-'
 	last if ( $line=~m/^\-\-\-\-\-\-\-+/o );
 
+	$potentialEntries++;
+	    
         # e.g. New  Distribution  Votes  Rank  Title
         #            0000000133  225568   8.9  12 Angry Men (1957)
 	if ( $line=~m/^\s+([\.|\*|\d]+)\s+(\d+)\s+(\d+\.\d+)\s+(.+)$/o ) {
@@ -1013,31 +1022,32 @@ sub importRatings($$)
 		}
 	    }
 	    
-	}
-	$count++;
+	    $count++;
 
-	#$self->{movies}{$line}=[$1,$2,"$3.$4"];
-	if ( $self->{showProgressBar} ) {
-	    # re-adjust target so progress bar doesn't seem too wonky
-	    if ( $count > $countEstimate ) {
-		$countEstimate = $progress->target($count+1000);
-		$next_update=$progress->update($count);
-	    }
-	    elsif ( $count > $next_update ) {
-		$next_update=$progress->update($count);
+	    #$self->{movies}{$line}=[$1,$2,"$3.$4"];
+	    if ( $self->{showProgressBar} ) {
+		# re-adjust target so progress bar doesn't seem too wonky
+		if ( $count > $countEstimate ) {
+		    $countEstimate = $progress->target($count+1000);
+		    $next_update=$progress->update($count);
+		}
+		elsif ( $count > $next_update ) {
+		    $next_update=$progress->update($count);
+		}
 	    }
 	}
 	else {
 	    $self->error("$file:$lineCount: unrecognized format");
-	    $next_update=$progress->update($count) if ($self->{showProgressBar});
 	}
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Ratings found ".withThousands($count)." in ".
-			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
     
     closeMaybeGunzip($file, $fh);
+    
+    $DB->commit();
     return($count);
 }
 
@@ -1049,7 +1059,7 @@ sub importKeywords($$$$)
 
     if ( $DB->table_row_count('Keywords') > 0 ) {
 	$self->status("clearing previously loaded data..");
-	$DB->runSQL("DELETE from Keywords");
+	$DB->table_clear('Keywords');
     }
 
     my $fh = openMaybeGunzip($file) || return(-2);
@@ -1069,7 +1079,7 @@ sub importKeywords($$$$)
 	    }
 	    last;
 	}
-	elsif ( $lineCount > 70000 ) {
+	elsif ( $lineCount > 200000 ) {
 	    $self->error("$file: stopping at line $lineCount, didn't see \"THE KEYWORDS LIST\" line");
 	    closeMaybeGunzip($file, $fh);
 	    return(-1);
@@ -1088,38 +1098,41 @@ sub importKeywords($$$$)
     my $count=0;
     my $countImported=0;
     my %found;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Keywords VALUES (?,?)');
-    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Keywords VALUES (?,?)');
+    my $tableInsert_sth1=$DB->prepare('INSERT INTO Keywords (KeywordID, Name) VALUES (?,?)');
+    my $tableInsert_sth2=$DB->prepare('INSERT INTO Titles2Keywords (TitleID, KeywordID) VALUES (?,?)');
     my $keywordCount=0;
+    my $potentialEntries=0;
+    
     while(<$fh>) {
 	$lineCount++;
 	my $line=$_;
 	chomp($line);
 	next if ($line =~ m/^\s*$/);
+	
+	$potentialEntries++;
+	
 	my ($title, $keyword) = ($line =~ m/^(.*)\s+(\S+)\s*$/);
 	if ( defined($title) and defined($keyword) ) {
 
             my ($episode) = $title =~ m/\s+(\{.*\})$/o;
             
-            # ignore anything which is an episode (e.g. "{Doctor Who (#10.22)}" )
-            #if ( !defined $episode || $episode eq '' )
-            #{
-		if ( $self->{imdbMovie2DBKey}->{$title} ) {
-		    if ( !defined($found{$keyword}) ) {
-			$found{$keyword}=$keywordCount++;
-			$tableInsert_sth->execute($keywordCount, $keyword);
-			#=$DB->insert_row('Keywords', 'KeywordID', Name=>$keyword);
-		    }
-		    $tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$title}, $found{$keyword});
-
-		    #$DB->insert_row('Titles2Keywords', undef, TitleID=>$self->{imdbMovie2DBKey}->{$title}, KeywordID=>$found{$keyword});
-		    $countImported++;
-		    if ( ($countImported % 50000) == 0 ) {
-			$DB->commit();
-		    }
+	    if ( $self->{imdbMovie2DBKey}->{$title} ) {
+		if ( !defined($found{$keyword}) ) {
+		    $keywordCount++;
+		    
+		    $found{$keyword}=$keywordCount;
+		    $tableInsert_sth1->execute($keywordCount, $keyword);
+		    #=$DB->insert_row('Keywords', 'KeywordID', Name=>$keyword);
 		}
-		$count++;
-	    #}
+		$tableInsert_sth2->execute($self->{imdbMovie2DBKey}->{$title}, $found{$keyword});
+		
+		#$DB->insert_row('Titles2Keywords', undef, TitleID=>$self->{imdbMovie2DBKey}->{$title}, KeywordID=>$found{$keyword});
+		$countImported++;
+		if ( ($countImported % 50000) == 0 ) {
+		    $DB->commit();
+		}
+	    }
+	    $count++;
 	    if ( $self->{showProgressBar} ) {
 		# re-adjust target so progress bar doesn't seem too wonky
 		if ( $count > $countEstimate ) {
@@ -1131,17 +1144,17 @@ sub importKeywords($$$$)
 		}
 	    }
         } else {
-	    #$self->error("$file:$lineCount: unrecognized format \"$line\"");
-	    $next_update=$progress->update($count) if ($self->{showProgressBar});
+	    $self->error("$file:$lineCount: unrecognized format \"$line\"");
 	}
 	
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Keywords found ".withThousands($count)." in ".
-			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
+    $DB->commit();
     return($count);
 }
 
@@ -1153,7 +1166,7 @@ sub importPlots($$$$)
 
     if ( $DB->table_row_count('Plots') > 0 ) {
 	$self->status("clearing previously loaded data..");
-	$DB->runSQL("DELETE from Plots");
+	$DB->table_clear('Plots');
     }
 
     my $fh = openMaybeGunzip($file) || return(-2);
@@ -1190,44 +1203,69 @@ sub importPlots($$$$)
     my $next_update=0;
 
     my $count=0;
-    my $tableInsert_sth=$DB->prepare('INSERT INTO Plots VALUES (?,?)');
+    my $potentialEntries=0;
+    my $tableInsert_sth=$DB->prepare('INSERT INTO Plots (TitleID, Sequence, Description, Author) VALUES (?,?,?,?)');
     while(<$fh>) {
 	$lineCount++;
 	my $line=$_;
 	chomp($line);
 	next if ($line =~ m/^\s*$/);
 	next if ($line=~m/\s*\{\{SUSPENDED\}\}/o);
+	
+	$potentialEntries++;
 
 	my ($title, $episode) = ($line =~ m/^MV:\s(.*?)\s?(\{.*\})?$/);
 	if ( defined($title) ) {
             
 	    $line =~s/^MV:\s*//;
 
+	    my $sequence=1;
 	    my $plot = '';
-	  LOOP:
-	    while (1) {
-		if ( my $l = <$fh> ) {
-		    $lineCount++;
-		    chomp($l);
-		    next if ($l =~ m/^\s*$/);
-		    if ( $l =~ m/PL:\s(.*)$/ ) {     # plot summary is a number of lines starting "PL:"
-			$plot .= ($plot ne ''?' ':'') . $1;
+
+	    while ( my $l = <$fh> ) {
+		$lineCount++;
+		chomp($l);
+		
+		next if ($l =~ m/^\s*$/);
+		
+		if ( $l =~ m/PL:\s(.*)$/ ) {     # plot summary is a number of lines starting "PL:"
+		    $plot .= ($plot ne '' ?' ':'') . $1;
+		}
+
+		if ( $l =~ m/BY:\s(.*)$/ || $l =~ m/^(\-\-\-\-\-\-\-\-)/o ) {
+		    my $token=$1;
+		    my $author=$1;
+		    
+		    if ( $token eq "\-\-\-\-\-\-\-\-" ) {
+			if ( $plot eq '' ) {
+			    last;
+			}
+			$author='';
+		    }
+		    
+		    if ( $self->{imdbMovie2DBKey}->{$line} ) {
+			$tableInsert_sth->execute($self->{imdbMovie2DBKey}->{$line}, $sequence, $plot, $author);
+			
+			$count++;
+			if ( ($count % 50000) == 0 ) {
+			    $DB->commit();
+			}
+		    } 
+		    else {
+			$self->error("$lineCount: unable to match title key '$line'");
+		    }
+		    
+		    $plot='';
+		    $sequence++;
+
+		    if ( $token eq "\-\-\-\-\-\-\-\-" ) {
+			last;
 		    }
-		    last LOOP if ( $l =~ m/BY:\s(.*)$/ );     # the author line "BY:" signals the end of the plot summary
-		} else {
-		    last LOOP;
 		}
 	    }
-	    
-	    if ( $self->{imdbMovie2DBKey}->{$line} ) {
-		$tableInsert_sth->execute($self->{imdbMovie2DBKey}->{$line}, $plot);
-		$count++;
-		if ( ($count % 50000) == 0 ) {
-		    $DB->commit();
-		}
-	    } 
-	    else {
-		#warn "$line";
+
+	    if ( length($plot) ) {
+		$self->error("$lineCount: truncated plot with title key '$line'");
 	    }
 	    
 	    if ( $self->{showProgressBar} ) {
@@ -1245,15 +1283,22 @@ sub importPlots($$$$)
             if ($line !~ m/^(---|PL:|BY:)/ ) {
                 $self->error("$file:$lineCount: unrecognized format \"$line\"");
             }
-	    $next_update=$progress->update($count) if ($self->{showProgressBar});
+	    #$next_update=$progress->update($count) if ($self->{showProgressBar});
+	    if ( $count > $next_update ) {
+		if ($self->{showProgressBar}) {
+		    $next_update=$progress->update($count) ;
+		    warn "next $count -> $next_update";
+		}
+	    }
 	}
     }
     $progress->update($countEstimate) if ($self->{showProgressBar});
 
     $self->status(sprintf("importing Plots found ".withThousands($count)." in ".
-			  withThousands($lineCount)." lines in %d seconds",time()-$startTime));
+			  withThousands($potentialEntries)." entries in %d seconds",time()-$startTime));
 
     closeMaybeGunzip($file, $fh);
+    $DB->commit();
     return($count);
 }
 
@@ -1320,6 +1365,10 @@ sub dbinfoGetFileSize($$)
 	die ("invalid call for $key");
     }
     my $filePath=$self->{listFiles}->paths_index($key);
+    if ( ! -f $filePath ) {
+	return(0);
+    }
+
     my $fileSize=int(-s $filePath);
 
     # if compressed, then attempt to run gzip -l
@@ -1399,7 +1448,7 @@ sub _prepStage
     }
    
     if ( !$DB->connect() ) {
-	die "moviedb connect failed:$DBI::errstr";
+	die "imdbdb connect failed:$DBI::errstr";
     }
     $DB->runSQL("PRAGMA synchronous = OFF");
     return($DB);
@@ -1435,14 +1484,13 @@ sub importListFile($$$)
 				  genres=>38,
 				  ratings=>68,
 				  keywords=>47,
-				  plot=>804);
+				  plot=>731);
 	my $fileSize=$self->dbinfoGetFileSize($key);
 	
 	my $countEstimate=int($fileSize/$estimateSizePerEntry{$key});
 	
 	my $filePath=$self->{listFiles}->paths_index($key);
 	
-	
 	$self->dbinfoAdd($key."_list_file", $filePath);
 	$self->dbinfoAdd($key."_list_file_size", "".int(-s $filePath));
 	$self->dbinfoAdd($key."_list_file_size_uncompressed", $fileSize);
@@ -1457,7 +1505,15 @@ sub importListFile($$$)
     };
 
 
+    if ( ! -f $self->{listFiles}->paths_index($type) ) {
+	$self->status("no $type file available");
+	return(1);
+    }
+
     if ( $type eq 'movies') {
+
+	$DB->drop_table_indexes('Titles');
+	
 	my $countEstimate=&$dbinfoCalcEstimate($self, "movies");
 
 	my $num=$self->importMovies($countEstimate, $self->{listFiles}->paths_index('movies'), $DB);
@@ -1479,6 +1535,10 @@ sub importListFile($$$)
 	close(OUT);
 
 	$self->dbinfoAdd("db_stat_movie_count", "$num");
+
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Titles');
+
 	return(0);
     }
 
@@ -1501,6 +1561,8 @@ sub importListFile($$$)
     # need to read-movie kesy
     if ( $type eq 'directors') {
 
+	$DB->drop_table_indexes('Directors');
+	
 	my $countEstimate=&$dbinfoCalcEstimate($self, "directors");
 
 	my $num=$self->importDirectors($countEstimate, $self->{listFiles}->paths_index('directors'), $DB);
@@ -1515,10 +1577,15 @@ sub importListFile($$$)
 	    $self->status("ARG estimate of $countEstimate for directors needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_director_count", "$num");
+	
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Directors');
+
 	return(0);
     }
 
     if ( $type eq 'actors') {
+	$DB->drop_table_indexes('Actors');
 
 	#print "re-reading movies into memory for reverse lookup..\n";
 	my $countEstimate=&$dbinfoCalcEstimate($self, "actors");
@@ -1536,7 +1603,6 @@ sub importListFile($$$)
 	    $self->status("ARG estimate of $countEstimate for actors needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_actor_count", "$num");
-
 	return(0);
     }
     
@@ -1556,10 +1622,15 @@ sub importListFile($$$)
 	}
 	$self->dbinfoAdd("db_stat_actress_count", "$num");
 	
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Actors');
+	
 	return(0);
     }
     
     if ( $type eq 'genres') {
+	$DB->drop_table_indexes('Genres');
+	
 	my $countEstimate=&$dbinfoCalcEstimate($self, "genres");
 
 	my $num=$self->importGenres($countEstimate, $self->{listFiles}->paths_index('genres'), $DB);
@@ -1574,11 +1645,16 @@ sub importListFile($$$)
 	    $self->status("ARG estimate of $countEstimate for genres needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_genres_count", "$num");
+	
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Genres');
 
 	return(0);
     }
     
     if ( $type eq 'ratings') {
+	$DB->drop_table_indexes('Ratings');
+	
 	my $countEstimate=&$dbinfoCalcEstimate($self, "ratings");
 
 	my $num=$self->importRatings($countEstimate, $self->{listFiles}->paths_index('ratings'), $DB);
@@ -1594,15 +1670,15 @@ sub importListFile($$$)
 	}
 	$self->dbinfoAdd("db_stat_ratings_count", "$num");
 
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Ratings');
+
 	return(0);
     }
     
     if ( $type eq 'keywords') {
+	$DB->drop_table_indexes('Keywords');
 	
-	if ( ! -f $self->{listFiles}->paths_index('keywords') ) {
-	    $self->status("no keywords file downloaded, see --with-keywords details in documentation");
-	    return(0);
-	}
 	my $countEstimate=&$dbinfoCalcEstimate($self, "keywords");
 	#my $countEstimate=5554178;
 
@@ -1620,15 +1696,14 @@ sub importListFile($$$)
 	$self->dbinfoAdd("keywords_list_file_size", -s $self->{listFiles}->paths_index('keywords'));
 	$self->dbinfoAdd("db_stat_keywords_count", "$num");
 
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Keywords');
+
 	return(0);
     }
 
     if ( $type eq 'plot') {
-	
-	if ( ! -f $self->{listFiles}->paths_index('plot') ) {
-	    $self->status("no plot file downloaded, see --with-plot details in documentation");
-	    return(0);
-	}
+	$DB->drop_table_indexes('Plots');
 	
 	my $countEstimate=&$dbinfoCalcEstimate($self, "plot");
 	my $num=$self->importPlots($countEstimate, $self->{listFiles}->paths_index('plot'), $DB);
@@ -1645,6 +1720,9 @@ sub importListFile($$$)
 	$self->dbinfoAdd("plots_list_file_size", -s $self->{listFiles}->paths_index('plot'));
 	$self->dbinfoAdd("db_stat_plots_count", "$num");
 	
+	$self->status("Creating Table indexes..");
+	$DB->create_table_indexes('Plots');
+	
 	return(0);
     }
 
@@ -1680,6 +1758,26 @@ sub importList($$)
     return(0);
 }
 
+=head2 importAll
+
+Import all available list files from 'listsDir' into the IMDB::Local Database.
+Returns # of list files that produced errors.
+
+=cut
+
+sub importAll($$)
+{
+    my ($self, $type)=@_;
+
+    my $err=0;
+    for my $type ( $self->listTypes() ) {
+        if ( $self->importList($type) != 0 ) {
+           warn("list import $type failed to load, $self->{errorCountInLog} errors in $self->{imdbDir}/stage-$type.log");
+	   $err++;
+        }
+    }
+    return($err);
+}
 
 sub _NOT_USED_checkSantity($)
 {
